/*
 * Created on 30.11.2004
 *
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package edu.teco.particles.pComponent.console;

import javax.swing.JPanel;

import java.awt.BorderLayout;

import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;
import javax.swing.table.*;

import java.awt.*;

/**
 * This is the GUI of the Console Particle Component.
 * 
 * @author scholl <scholl@teco.edu>
 */
public class ConsoleGUI extends JPanel
{

  private JTable      jTable      = null;

  /**
   * 
   * @uml.property name="jScrollPane"
   */
  private JScrollPane jScrollPane = null;

  /**
   * This is the default constructor
   */
  public ConsoleGUI()
  {
    super();
    initialize();
  }

  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize()
  {
    this.setLayout(new BorderLayout());
    this.setSize(300, 200);
    this.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
  }

  /**
   * set the Model. removes the first column with the assumption that it
   * contains the packet Number.
   * 
   * @param model
   */
  public void setModel(TableModel model)
  {
    jTable.setModel(model);
    jTable.createDefaultColumnsFromModel();
    jTable.removeColumn(jTable.getColumnModel().getColumn(0));
  }

  public TableColumnModel getColumnModel()
  {
    return jTable.getColumnModel();
  }

  /**
   * This method initializes jTable
   * 
   * @return javax.swing.JTable
   * 
   * @uml.property name="jTable"
   */

  private JTable getJTable()
  {
    if (jTable == null)
    {
      jTable = new JTable();
      jTable.setShowVerticalLines(false);
      jTable.setRowSelectionAllowed(false);
      jTable.setColumnSelectionAllowed(false);
      jTable.setDefaultRenderer(String.class, new ColorTableRenderer(new Color(
          255, 252, 179)));
      jTable.setDefaultRenderer(String[].class, new ColorTableRenderer(
          new Color(255, 252, 179)));
      jTable.setAutoCreateColumnsFromModel(false);
      jTable.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
      
      // performance fix, see swing table faq
      ToolTipManager.sharedInstance().unregisterComponent(jTable);
      ToolTipManager.sharedInstance().unregisterComponent(jTable.getTableHeader());
    }
    return jTable;
  }

  /**
   * This method initializes jScrollPane
   * 
   * @return javax.swing.JScrollPane
   * 
   * @uml.property name="jScrollPane"
   */
  private JScrollPane getJScrollPane()
  {
    if (jScrollPane == null)
    {
      jScrollPane = new JScrollPane();
      jScrollPane.setViewportView(getJTable());
    }
    return jScrollPane;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.SettingsListener#settingsChanged()
   */
  public void settingsChanged()
  {
    // do nothing
  }
}
