package edu.teco.particles.pComponent.otDebug;

import java.io.*;
import java.nio.channels.FileChannel;

public class CODFile extends FileInputStream
{

  public CODFile(File f) throws FileNotFoundException
  {
    super(f);
    this.c = getChannel();
  }

  FileChannel c;

  class Table
  {
    int startBlock;

    int endBlock;

    private Table(Table t)
    {
      this.startBlock = t.startBlock;
      this.endBlock = t.endBlock;
    }

    private Table(int start, int end)
    {
      this.startBlock = start;
      this.endBlock = end;
    }

    final static short BLOCK_SIZE = (1 << 9);

    protected void check(int offset) throws IOException
    {
      if ((offset / BLOCK_SIZE) > (endBlock - startBlock))
        throw (new IOException("offset too big"));
    }

    int tell() throws IOException
    {
      return (int) (c.position() - (startBlock * BLOCK_SIZE));
    }

    void reset() throws IOException
    {
      seek(0);
    }

    void seek(int offset) throws IOException
    {
      check(offset);
      c.position((startBlock * BLOCK_SIZE) + offset);
    }

    String readString(int size) throws IOException
    {
      byte[] b = new byte[size];
      int offset = tell();
      if ((BLOCK_SIZE - (offset % BLOCK_SIZE)) < size)
        skip((BLOCK_SIZE - (offset % BLOCK_SIZE)));

      check(tell());
      read(b);
      if (b[0] == 0) return null;
      return new String(b, 1, b[0]);
    }
  };

  class Lines extends Table
  {
    class Line
    {
      int              file;

      int              flags;

      int              number;

      int              address;

      short            srcLine;

      static final int length = 6;

      public String toString()
      {
        try
        {
          return file + ":" + number + "@0x" + Integer.toHexString(address)
              + "(0x" + Integer.toHexString(flags) + ")";
        }
        catch (Exception E)
        {
          return null;
        }
      }
    }

    Line readLine() throws IOException
    {
      byte buf[] = new byte[Line.length];
      Line l = new Line();

      int offset = tell();

      if ((BLOCK_SIZE - (offset % BLOCK_SIZE)) < Line.length)
        skip((BLOCK_SIZE - (offset % BLOCK_SIZE)));

      check(tell());

      read(buf);

      l.file = buf[0] & 0xff;
      l.flags = buf[1] & 0xff;
      l.number = buf[2] & 0xff | ((buf[3] & 0xff) << 8);
      l.address = buf[4] & 0xff | ((buf[5] & 0xff) << 8);

      return l;
    }

    public Lines() throws IOException
    {
      super(0, 0);
      Directory d = new Directory();
      d.seek(CODFile.Directory.Position.LSTTAB);
      Table t = d.readTable();
      startBlock = t.startBlock;
      endBlock = t.endBlock;

    }

    Line getLine(int n) throws IOException
    {
      int offset = (n * Line.length)
          + ((n / (BLOCK_SIZE / Line.length)) * (BLOCK_SIZE % Line.length)); // correct
      // the
      // 512/6
      // Problem
      // TODO:
      // probably
      // applies
      // to
      // all
      // so
      // move
      // into
      // generic
      // interface
      seek(offset);
      Line l = readLine();
      return l;
    }
  };

  class Names extends Table
  {
    public Names() throws IOException
    {
      super(0, 0);
      Directory d = new Directory();
      d.seek(CODFile.Directory.Position.NAMTAB);
      Table t = d.readTable();
      startBlock = t.startBlock;
      endBlock = t.endBlock;

    }

    static final int STRING_SIZE = 64;

    String readName() throws IOException
    {
      return readString(STRING_SIZE);
    }

    String getName(int n) throws IOException
    {
      seek(n * STRING_SIZE);
      return readName();
    }
  };

  class Directory extends Table
  {
    public Directory()
    {
      super((short) 0, (short) Table.BLOCK_SIZE);
    }

    final class Position
    {
      final static short CODE         = 0 /* code block indices are at the start */;

      final static short SOURCE       = 257 /* source file name String[55] */;

      final static short DATE         = 321 /*
                                             * date .cod file was created
                                             * String[8]
                                             */;

      final static short TIME         = 328 /*
                                             * time .cod file was created
                                             * UInt16=h*100+m ???
                                             */;

      final static short VERSION      = 331 /* Compiler version String[10] */;

      final static short COMPILER     = 351 /* -Compiler name String[12] */;

      final static short NOTICE       = 363 /* Compiler copyright String[63] */;

      final static short SYMTAB       = 426 /*
                                             * Start block of short symbol table
                                             * UInt16
                                             */;

      final static short SYMTAB_END   = 428 /*
                                             * Start block of short symbol table
                                             * UInt16
                                             */;

      final static short NAMTAB       = 430 /*
                                             * Start block of file name table
                                             * UInt16
                                             */;

      final static short NAMTAB_END   = 432 /*
                                             * Start block of file name table
                                             * UInt16
                                             */;

      final static short LSTTAB       = 434 /*
                                             * Start block of list file cross
                                             * reference UInt16
                                             */;

      final static short LSTTAB_END   = 436 /*
                                             * End block of list file cross
                                             * reference UInt16
                                             */;

      final static short ADDRSIZE     = 438 /* # of bytes for an address Uint8 */;

      final static short HIGHADDR     = 439 /*
                                             * High word of address for 64K Code
                                             * block UInt16
                                             */;

      final static short NEXTDIR      = 441 /* Next directory block UInt16 */;

      final static short MEMMAP       = 443 /* Start block of memory map */;

      final static short MEMMAP_END   = 445 /* End block of memory map */;

      final static short LOCALVAR     = 447 /* Start block of local variables */;

      final static short LOCALVAR_END = 449 /* End block of local variables */;

      final static short CODTYPE      = 451 /* Type of .cod file UInt16 */;

      final static short PROCESSOR    = 454 /* Target processor String[8] */;

      final static short LSYMTAB      = 462 /* Start block of long symbol table */;

      final static short LSYMTAB_END  = 464 /* End block of long symbol table */;

      final static short MESSTAB      = 466 /* Start block of debug message area */;

      final static short MESSTAB_END  = 468 /* End block of debug message area */;
    }

    Table readTable() throws IOException
    {
      byte[] b = new byte[4];
      read(b);
      return new Table(b[0] & 0xff, b[2] & 0xff);
    }
  }

  class Line
  {
    short srcNr;

    byte  flags;

    short fileid;

  }
}
