/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.scope;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import edu.teco.particles.pComponent.util.ParticleContext;
import edu.teco.particles.pComponent.util.TupleParser;
import edu.teco.particles.pComponent.util.TupleParserFactory;

/**
 * A JComponent capsulating the Control and View to configure the drawing of a
 * ParticleContext Trace.
 */
public class ContextEditor extends JPanel
{
  protected ParticleContext _context;

  protected Trace           _trace;

  protected TupleParser     _parser;

  private JLabel            jLabel     = null;

  private JLabel            jLabel1    = null;

  private JComboBox         jComboBox  = null;

  private JComboBox         jComboBox1 = null;

  private JLabel            jLabel2    = null;

  private JLabel            jLabel3    = null;

  private JSpinner          jSpinner   = null;

  private JSpinner          jSpinner1  = null;

  private JLabel            jLabel4    = null;

  public ContextEditor(ParticleContext context)
  {
    jLabel = new JLabel();
    setContext(context);

    initialize();
  }

  /**
   * update class variables for new context
   * 
   * @param ctx
   *          the new context
   */
  public void setContext(ParticleContext ctx)
  {
    assert (ctx != null);

    _context = ctx;
    _trace = TraceFactory.getTrace(_context);
    _parser = TupleParserFactory.getTupleParser(_context);

    assert (_context.getColor().equals(_trace.getColor()));

    // updates their view
    getJSpinner();
    getJSpinner1();
    getJComboBox();
    getJComboBox1();
    getColorLabel();

    jLabel.setText("Configuration for: " + ctx.toString());
  }

  /** initiliazes the View of this Component */
  protected void initialize()
  {
    GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
    jLabel3 = new JLabel();
    jLabel2 = new JLabel();
    jLabel1 = new JLabel();
    GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
    GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
    GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
    GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
    GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
    GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
    GridBagConstraints gridBagConstraints18 = new GridBagConstraints();
    GridBagConstraints gridBagConstraints19 = new GridBagConstraints();
    this.setLayout(new GridBagLayout());
    this.setSize(451, 200);
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 0;
    gridBagConstraints1.gridwidth = 4;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints1.insets = new java.awt.Insets(0, 10, 10, 0);
    gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
    jLabel.setText("Configure: 127.123.123.12.312.31 SGX");
    gridBagConstraints2.gridx = 0;
    gridBagConstraints2.gridy = 1;
    gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints2.insets = new java.awt.Insets(0, 10, 10, 10);
    jLabel1.setText("parser:");
    gridBagConstraints4.gridx = 1;
    gridBagConstraints4.gridy = 1;
    gridBagConstraints4.weightx = 1.0;
    gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints4.insets = new java.awt.Insets(0, 10, 10, 10);
    gridBagConstraints5.gridx = 2;
    gridBagConstraints5.gridy = 1;
    gridBagConstraints5.weightx = 1.0;
    gridBagConstraints5.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints5.insets = new java.awt.Insets(0, 10, 10, 10);
    gridBagConstraints16.gridx = 0;
    gridBagConstraints16.gridy = 2;
    gridBagConstraints16.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints16.insets = new java.awt.Insets(0, 10, 10, 10);
    jLabel2.setText("Byte Offset:");
    gridBagConstraints17.gridx = 0;
    gridBagConstraints17.gridy = 3;
    gridBagConstraints17.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints17.insets = new java.awt.Insets(0, 10, 10, 10);
    jLabel3.setText("Draw Offset:");
    gridBagConstraints18.gridx = 1;
    gridBagConstraints18.gridy = 2;
    gridBagConstraints18.anchor = java.awt.GridBagConstraints.EAST;
    gridBagConstraints18.insets = new java.awt.Insets(0, 0, 0, 10);
    gridBagConstraints19.gridx = 1;
    gridBagConstraints19.gridy = 3;
    gridBagConstraints19.anchor = java.awt.GridBagConstraints.EAST;
    gridBagConstraints19.insets = new java.awt.Insets(0, 0, 0, 10);
    gridBagConstraints11.gridx = 2;
    gridBagConstraints11.gridy = 2;
    gridBagConstraints11.gridheight = 2;
    gridBagConstraints11.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints11.insets = new java.awt.Insets(10, 10, 10, 10);
    this.add(jLabel, gridBagConstraints1);
    this.add(jLabel1, gridBagConstraints2);
    this.add(getJComboBox(), gridBagConstraints4);
    this.add(getJComboBox1(), gridBagConstraints5);
    this.add(jLabel2, gridBagConstraints16);
    this.add(jLabel3, gridBagConstraints17);
    this.add(getJSpinner(), gridBagConstraints18);
    this.add(getJSpinner1(), gridBagConstraints19);
    this.add(getColorLabel(), gridBagConstraints11);
  }

  protected JLabel colorLabel;

  /**
   * @return
   */
  private JLabel getColorLabel()
  {
    if (colorLabel == null)
    {
      colorLabel = new JLabel();

      colorLabel.addMouseListener(new MouseAdapter() {
        public void mousePressed(MouseEvent e)
        {
          Color color = JColorChooser.showDialog(colorLabel,
              "choose trace color", _trace.getColor());
          _trace.setColor(color);
          _context.setColor(color);

          colorLabel.setBackground(_trace.getColor());
        }
      });
    }

    colorLabel.setBackground(_trace.getColor());
    colorLabel.setOpaque(true);

    return colorLabel;
  }

  protected static String SIGNED   = "signed";

  protected static String UNSIGNED = "unsigned";

  /**
   * This method initializes jComboBox
   * 
   * @return javax.swing.JComboBox
   */
  private JComboBox getJComboBox()
  {
    DefaultComboBoxModel model;

    if (jComboBox == null)
    {
      jComboBox = new JComboBox();
      jComboBox.setPreferredSize(new java.awt.Dimension(13, 18));

      model = (DefaultComboBoxModel) jComboBox.getModel();

      model.addElement(SIGNED);
      model.addElement(UNSIGNED);

      jComboBox.addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent arg0)
        {
          String value = (String) arg0.getItem();

          if (value.equals(SIGNED))
          {
            _parser.setSigned(true);
          }
          else
          {
            _parser.setSigned(false);
          }
        }
      });
    }

    model = (DefaultComboBoxModel) jComboBox.getModel();

    if (_parser.isSigned())
    {
      model.setSelectedItem(SIGNED);
    }
    else
    {
      model.setSelectedItem(UNSIGNED);
    }

    return jComboBox;
  }

  /**
   * This method initializes jComboBox1
   * 
   * @return javax.swing.JComboBox
   */
  private JComboBox getJComboBox1()
  {
    DefaultComboBoxModel model;

    if (jComboBox1 == null)
    {
      jComboBox1 = new JComboBox();
      jComboBox1.setPreferredSize(new java.awt.Dimension(32, 18));

      model = (DefaultComboBoxModel) jComboBox1.getModel();

      for (String type : TupleParserFactory.getAvailableTypes())
      {
        model.addElement(type);
      }

      jComboBox1.addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent arg0)
        {
          String value = (String) arg0.getItem();
          TupleParser oldParser = _parser;

          _parser = TupleParserFactory.getTupleParser(_context, value);

          _parser.setOffset(oldParser.getOffset());
          _parser.setSigned(oldParser.isSigned());
        }
      });
    }

    model = (DefaultComboBoxModel) jComboBox1.getModel();
    model.setSelectedItem(_parser.getDataType());

    return jComboBox1;
  }

  /**
   * This method initializes jSpinner
   * 
   * @return javax.swing.JSpinner
   */
  private JSpinner getJSpinner()
  {
    if (jSpinner == null)
    {
      jSpinner = new JSpinner();

      jSpinner.addChangeListener(new ChangeListener() {
        public void stateChanged(ChangeEvent arg0)
        {
          Integer value = (Integer) jSpinner.getValue();
          _parser.setOffset(value.intValue());
        }
      });
    }

    jSpinner.setValue(_parser.getOffset());
    jSpinner.setPreferredSize(new java.awt.Dimension(58, 20));

    return jSpinner;
  }

  /**
   * This method initializes jSpinner1
   * 
   * @return javax.swing.JSpinner
   */
  private JSpinner getJSpinner1()
  {
    if (jSpinner1 == null)
    {
      jSpinner1 = new JSpinner();

      jSpinner1.addChangeListener(new ChangeListener() {
        public void stateChanged(ChangeEvent arg0)
        {
          Integer value = (Integer) jSpinner1.getValue();
          _trace.setOffset(value.intValue());
        }
      });
    }

    jSpinner1.setValue(_trace.getOffset());
    jSpinner1.setPreferredSize(new java.awt.Dimension(58, 20));

    return jSpinner1;
  }
} // @jve:decl-index=0:visual-constraint="10,10"
