/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.scope;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;

/**
 * This is a Factory for Shapes built from a Trace.
 */
public class TraceShaper
{
  protected static GeneralPath impulseShape;

  protected static GeneralPath continiousShape;

  /**
   * creates a Shape that represents the trace as a list of bars.
   * 
   * @param trace
   *          the trace to be shaped
   * @param coordinate
   *          the coordinate system to draw on
   * @return a Bar Shape :)
   */
  public static Shape createImpulsesShape(Trace trace,
      Rectangle2D.Double coordinate, double res)
  {
    if (impulseShape == null)
    {
      impulseShape = new GeneralPath(GeneralPath.WIND_NON_ZERO);
    }
    else
    {
      impulseShape.reset();
    }

    Point2D pt;

    synchronized (trace)
    {
      Iterator it = trace.deltaIterator(res);

      while (it.hasNext())
      {
        pt = (Point2D) it.next();
        if (pt.getX() >= coordinate.getX())
        {
          impulseShape.moveTo((float) pt.getX(), (float) trace.getOffset());
          impulseShape.lineTo((float) pt.getX(), (float) pt.getY()
              + trace.getOffset());
        }
      }
    }

    return impulseShape;
  }

  /** creates a Shape that only contains the points */
  public static Shape createPointShape(Trace trace)
  {
    return null;
  }

  /**
   * @param trace
   * @return
   */
  public static Shape createContiniousShape(Trace trace,
      Rectangle2D.Double coordinate, double res)
  {
    if (continiousShape == null)
    {
      continiousShape = new GeneralPath(GeneralPath.WIND_NON_ZERO);
    }
    else
    {
      continiousShape.reset();
    }

    Point2D pt;
    boolean first;

    synchronized (trace)
    {
      first = true;
      Iterator it = trace.deltaIterator(res);

      while (it.hasNext())
      {
        pt = (Point2D) it.next();

        // XXX try high resolution on low resolution sending !
        if (first && pt.getX() >= coordinate.getX())
        {
          first = false;
          continiousShape.moveTo((float) pt.getX(), (float) pt.getY()
              + trace.getOffset());
        }
        else if (pt.getX() >= coordinate.getX())
        {
          continiousShape.lineTo((float) pt.getX(), (float) pt.getY()
              + trace.getOffset());
        }
      }
    }
    return continiousShape;
  }

}
