/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.sender;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import edu.teco.particles.libParticle.ParticlePacket;
import edu.teco.particles.pComponent.SocketThread;

/**
 * sends a packet over the socket.
 */
public class DefaultSendAction extends AbstractAction implements SendAction
{

  protected SocketThread _socket;

  /**
   * set the socket to send the packet over.
   */
  public void setSocket(SocketThread socket)
  {
    _socket = socket;
  }

  protected ProxyParticlePacket _pck;

  /**
   * set the packet to be send.
   */
  public void setProxyPacket(ProxyParticlePacket pck)
  {
    _pck = pck;
  }

  public DefaultSendAction(SocketThread sock, ProxyParticlePacket pck)
  {
    assert (pck != null) : " null packet set";

    _pck = pck;
    _socket = sock;
  }

  /**
   * request a particlePacket and send it.
   * 
   */
  public void actionPerformed(ActionEvent arg0)
  {
    ParticlePacket packet = _pck.getPacket();
    _socket.send(packet);
    _pck.setSeqNr((short) (_pck.getSeqNr() + 1));
  }

  /*
   * (non-Javadoc)
   * 
   * @see javax.swing.Action#getValue(java.lang.String)
   */
  public Object getValue(String key)
  {
    if (key.equals(AbstractAction.NAME))
    {
      return "Send";
    }

    return super.getValue(key);
  }
}
