/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.util;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;

import edu.teco.particles.libParticle.ParticleTuple;

/**
 * Parser for CTS tuple to Java Data Format.
 */
public class CTSParser implements TupleParser
{
  /** holds the mapping from srcid+acltuple to startTime. */
  protected HashMap<String, Date> _map = new HashMap<String, Date>();

  /**
   * parses a given context.
   * 
   * @param pt
   *          the tuple to ASSOCIATE the timeStamp with.
   * @param pck
   *          the packet in which the tuple is contained.
   */
  public Object parse(ParticleTuple pt) throws IllegalArgumentException
  {
    Calendar cal = Calendar.getInstance();
    short[] data = pt.toArray();

    cal.set(2000 + data[0], data[1], data[2], data[3], data[4], data[5]);
    if (data.length == 8) // milliseconds are available *happy* ;)
      cal.set(Calendar.MILLISECOND, (data[6] << 8 | data[7]));

    return _fmt.format(cal.getTime());
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#setOffset(int)
   */
  public void setOffset(int offset) throws IllegalArgumentException
  {
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#getOffset()
   */
  public int getOffset()
  {
    return 0;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#isSigned()
   */
  public boolean isSigned()
  {
    return false;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#setSigned(boolean)
   */
  public void setSigned(boolean signed)
  {
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#getDataType()
   */
  public String getDataType()
  {
    return "Control TimeStamp";
  }

  // ISO8601 date format with milliseconds.
  protected DateFormat _fmt = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");

  /**
   * delegate to Date.
   */
  public void setFormat(DateFormat fmt)
  {
    _fmt = fmt;
  }

  public short[] serialize(Object obj) throws IllegalArgumentException
  {
    short[] sarr = new short[0];

    if (obj instanceof Long)
    {
      Long num = (Long) obj;
      Date myDate = new Date(num);

      Calendar cal = Calendar.getInstance();
      cal.setTime(myDate);

      sarr = new short[8];
      sarr[0] = (short) (cal.get(Calendar.YEAR) - 2000);
      sarr[1] = (short) cal.get(Calendar.MONTH);
      sarr[2] = (short) cal.get(Calendar.DAY_OF_MONTH);
      sarr[3] = (short) cal.get(Calendar.HOUR_OF_DAY);
      sarr[4] = (short) cal.get(Calendar.MINUTE);
      sarr[5] = (short) cal.get(Calendar.SECOND);
      sarr[6] = (short) (cal.get(Calendar.MILLISECOND) >> 8);
      sarr[7] = (short) (cal.get(Calendar.MILLISECOND) & 0x00FF);
    }
    else if (obj instanceof String)
    {
      // ignore.
      return new short[] {};
    }
    else
    {
      throw new IllegalArgumentException("no serialization for type "
          + obj.getClass());
    }

    return sarr;
  }
}
