/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.util;

import edu.teco.particles.libParticle.ParticleTuple;

/**
 * Parser for one byte, little endian, signed/unsigned to Java Short.
 */
public class ShortParser implements TupleParser
{

  public ShortParser()
  {
    setOffset(0);
    setSigned(false);
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#parse(edu.teco.particles.libParticle.ParticleTuple)
   */
  public Object parse(ParticleTuple pt) throws IllegalArgumentException
  {
    if (pt.length() <= getOffset())
    {
      throw new IllegalArgumentException("ParticleTuple " + pt.getAclType()
          + " " + pt.toString() + " is too short for offset" + getOffset());
    }

    short value = 0;

    if (isSigned())
    {
      value = (byte) pt.getAclByte(getOffset());
    }
    else
    {
      try
      {
        value = pt.getAclByte(getOffset());
      }
      catch (IllegalArgumentException e)
      {
        System.out.println("index out of range " + getOffset() + " on tuple "
            + pt.getAclType());
      }
    }

    return new Short(value);
  }

  protected int _offset;

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#setOffset(int)
   */
  public void setOffset(int offset)
  {
    assert (offset >= 0);
    _offset = offset;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#getOffset(int)
   */
  public int getOffset()
  {
    return _offset;
  }

  protected boolean _signed;

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#isSigned()
   */
  public boolean isSigned()
  {
    return _signed;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#setSigned(boolean)
   */
  public void setSigned(boolean signed)
  {
    _signed = signed;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#getDataType()
   */
  public String getDataType()
  {
    return "Short (8bit)";
  }

  /**
   * tries to make a short[] out of a object.
   * 
   * @param obj
   *          the object to serialize.
   * @return a short array with the serialized data.
   */
  public short[] serialize(Object obj) throws IllegalArgumentException
  {
    short[] sarr = null;

    if (obj == null)
      return new short[0];
    else if (obj instanceof String)
    {
      String str = (String) obj;
      String[] strarr = str.split(" ");

      if (strarr[0] != "")
      {
        sarr = new short[strarr.length];
        for (int i = 0; i < strarr.length; i++)
        {
          sarr[i] = Short.parseShort(strarr[i]);
        }
      }
      else
      {
        return new short[] {};
      }
    }
    else if (obj instanceof Integer)
    {
      Integer num = (Integer) obj;
      sarr = new short[] { 0 };
      sarr[0] |= num.byteValue();
    }
    else
    {
      throw new IllegalArgumentException("No parser for object of type "
          + obj.getClass());
    }

    return sarr;
  }
}
