/*
 * Created on 25.01.2005
 *
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package edu.teco.particles.pComponent.util;

import edu.teco.particles.libParticle.ParticleTuple;

/**
 * Interface for a TupleParser.
 */
public interface TupleParser
{
  /**
   * parses the ParticleTuple.
   * 
   * @param pt
   *          the tuple to parse.
   * @return the parsed data or null if the packet can't be parsed.
   */
  public Object parse(ParticleTuple pt) throws IllegalArgumentException;

  /**
   * the counterpart of parsing.
   * 
   * @param obj
   *          the object to serialize.
   * @return a short[] array with the serialized object.
   * 
   * @throws IllegalArgumentException
   *           if the obj couldn't be serialized.
   */
  public short[] serialize(Object obj) throws IllegalArgumentException;

  /**
   * set the Offset from where to start parsing the data.
   * 
   * @param offset
   *          a positive int
   */
  public void setOffset(int offset);

  /**
   * getter for the offset
   */
  public int getOffset();

  /**
   * getter for the signed value.
   * 
   * @return true if the parsed data should be signed.
   */
  public boolean isSigned();

  /**
   * set wether the parser should create signed or unsigned values.
   */
  public void setSigned(boolean signed);

  /**
   * returns a short description of the parsed Type
   * 
   * @return String the dataType
   */
  public String getDataType();
}
